{% extends "logged_in_base.html" %}

{% block app_specific_sidebar %}
{% include sidebar_template %}
{% endblock %}

{% block content %}
  <div id="notfound">
    <div class="notfound">
      <div class="notfound-404">
        <h1>Oops!</h1>
        <h3>{{ error_message}}</h3>
      </div>
      <a href="/{{ dashboard_prefix }}/contest">Go To Homepage</a>
    </div>
  </div>
{% endblock %}