<head>
  <!-- Google font -->
  <link href="https://fonts.googleapis.com/css?family=Montserrat:200,400,700" rel="stylesheet">

  <!-- Custom stlylesheet -->
  <link type="text/css" rel="stylesheet" href="{{dashboard_assets_prefix}}/css/local/style.css" />
</head>

<body>

  <div id="notfound">
    <div class="notfound">
      <div class="notfound-404">
        <h1>Oops!</h1>
        <h2>{{ error_message}}</h2>
      </div>
      <a href={{ homepage }}>Go To Homepage</a>
    </div>
  </div>